import { Host, forceUpdate, h } from "@stencil/core";
import "igniteui-trial-watermark";
import { resourceStringsMap } from "../../utils/locale";
import { TwoWayMap } from "../../utils/two-way-map";
import { Utils } from "../../utils/utils";
import { IGC_DEFAULT_PANE_SIZE, IGC_DEFAULT_UNPIN_PANE_SIZE, IgcContextMenuOrientation, IgcTabHeadersPosition } from "./dockmanager.interfaces";
import { IgcDockManagerPaneType, IgcDockingIndicatorPosition, IgcSplitPaneOrientation, IgcUnpinnedLocation, IgcPaneActionBehavior } from "./dockmanager.public-interfaces";
import { IgcDockManagerService } from "./dockmanager.service";
import { IgcDockManagerKeyboardService } from "./keyboard/keyboard.service";
var ActionReason;
(function (ActionReason) {
  ActionReason["click"] = "click";
  ActionReason["drop"] = "drop";
  ActionReason["maximizeOrMinimize"] = "maximizeOrMinimize";
})(ActionReason || (ActionReason = {}));
const PANE_HEADER_HEIGHT = 40;
/**
 * @hidden
 */
export class IgcDockManager {
  constructor() {
    this._tabHeaderIconClicked = false;
    this.service = new IgcDockManagerService(this);
    this.keyboardService = new IgcDockManagerKeyboardService(this.service);
    this.activePaneInternalSet = false;
    this.domObserver = new MutationObserver(this.mutationCallback.bind(this));
    this.shouldMovePane = false;
    this.scheduledCallbacks = [];
    this.shouldClearActivePane = true;
    this.isDragging = false;
    this.splitterOffset = 1;
    this.loaded = false;
    this.resizeObserver = new ResizeObserver(this.restrictFloatingPaneSize.bind(this));
    this.proximityDockThreshold = 50;
    this.proximityOuterDockThreshold = 25;
    this.anyPaneIgnoringResizeConstraints = false;
    this.templatableComponents = [
      {
        slot: 'paneHeaderCloseButton',
        targetName: 'igc-pane-header-component',
        targetSlot: 'paneHeaderCloseButton',
        targetPart: 'pane-header-close-button'
      },
      {
        slot: 'tabHeaderCloseButton',
        targetName: 'igc-tab-header-component',
        targetSlot: 'tabHeaderCloseButton',
        targetPart: 'tab-header-close-button'
      },
      {
        slot: 'tabHeaderCloseButton',
        targetName: 'igc-context-menu-component',
        targetSlot: 'contextMenuCloseButton',
        targetPart: 'context-menu-close-button'
      },
      {
        slot: 'closeButton',
        targetName: 'igc-pane-header-component',
        targetSlot: 'paneHeaderCloseButton',
        targetPart: 'pane-header-close-button'
      },
      {
        slot: 'closeButton',
        targetName: 'igc-tab-header-component',
        targetSlot: 'tabHeaderCloseButton',
        targetPart: 'tab-header-close-button'
      },
      {
        slot: 'closeButton',
        targetName: 'igc-context-menu-component',
        targetSlot: 'contextMenuCloseButton',
        targetPart: 'context-menu-close-button'
      },
      {
        slot: 'moreTabsButton',
        targetName: 'igc-tabs-component',
        targetSlot: 'tabsMoreButton',
        targetPart: 'tabs-more-button'
      },
      {
        slot: 'maximizeButton',
        targetName: 'igc-pane-header-component',
        targetSlot: 'paneHeaderMaximizeButton',
        targetPart: 'pane-header-maximize-button'
      },
      {
        slot: 'maximizeButton',
        targetName: 'igc-tabs-component',
        targetSlot: 'tabsMaximizeButton',
        targetPart: 'tabs-maximize-button'
      },
      {
        slot: 'minimizeButton',
        targetName: 'igc-pane-header-component',
        targetSlot: 'paneHeaderMinimizeButton',
        targetPart: 'pane-header-minimize-button'
      },
      {
        slot: 'minimizeButton',
        targetName: 'igc-tabs-component',
        targetSlot: 'tabsMinimizeButton',
        targetPart: 'tabs-minimize-button'
      },
      {
        slot: 'pinButton',
        targetName: 'igc-pane-header-component',
        targetSlot: 'paneHeaderPinButton',
        targetPart: 'pane-header-pin-button'
      },
      {
        slot: 'unpinButton',
        targetName: 'igc-pane-header-component',
        targetSlot: 'paneHeaderUnpinButton',
        targetPart: 'pane-header-unpin-button'
      },
      {
        slot: 'unpinButton',
        targetName: 'igc-context-menu-component',
        targetSlot: 'contextMenuUnpinButton',
        targetPart: 'context-menu-unpin-button'
      },
      {
        slot: 'moreOptionsButton',
        targetName: 'igc-tab-header-component',
        targetSlot: 'tabHeaderMoreOptionsButton',
        targetPart: 'tab-header-more-options-button'
      },
      {
        slot: 'splitterHandle',
        targetName: 'igc-splitter-component',
        targetSlot: 'splitterHandle',
        targetPart: 'splitter-handle'
      },
    ];
    this.updateTemplates = () => {
      if (this.templates.size > 0) {
        return;
      }
      const slotted = this.element.shadowRoot.querySelectorAll('slot');
      const slottedArray = Array.from(slotted);
      const usedTemplates = new Map();
      const templatableSlots = this.templatableComponents.map(comp => comp.slot);
      templatableSlots.forEach(slot => {
        const matchedSlots = slottedArray.filter((s) => s.name === slot);
        if (!this.templates.get(slot) && matchedSlots.length > 0 && matchedSlots[0].assignedElements().length > 0) {
          usedTemplates.set(slot, matchedSlots[0].assignedElements()[0]);
        }
      });
      this.templates = usedTemplates;
    };
    this.handleCustomContentFocusIn = (eventArgs) => {
      const parent = eventArgs.currentTarget;
      if (parent) {
        const contentPane = this.service.clientContentPanesMap.get(parent.slot);
        if (!contentPane) {
          return;
        }
        this.setActivePane(contentPane);
      }
    };
    this._isValidDrop = true;
    this.handleShadowRootFocusOut = () => {
      if (this.paneToFocus) {
        return;
      }
      const rootNode = this.getRootNode();
      requestAnimationFrame(() => {
        if (!this.element.shadowRoot.activeElement && rootNode.activeElement === this.element && this.reason !== ActionReason.maximizeOrMinimize) {
          this.clearActivePane();
        }
      });
    };
    this.dropTargetPaneInfo = undefined;
    this.flyoutPane = undefined;
    this.floatingPaneZIndicesMap = new Map();
    this.contextMenuMeta = undefined;
    this.dropShadowRect = undefined;
    this.templates = new Map();
    this.hasCustomMaximizeButton = undefined;
    this.hasCustomMinimizeButton = undefined;
    this.hoveredPane = undefined;
    this.navigationPaneMeta = undefined;
    this.allowMaximize = true;
    this.showPaneHeaders = 'always';
    this.showHeaderIconOnHover = 'none';
    this.maximizedPane = undefined;
    this.resourceStrings = undefined;
    this.allowFloatingPanesResize = true;
    this.disableKeyboardNavigation = false;
    this.contextMenuPosition = undefined;
    this.containedInBoundaries = false;
    this.allowInnerDock = true;
    this.proximityDock = false;
    this.closeBehavior = IgcPaneActionBehavior.allPanes;
    this.unpinBehavior = IgcPaneActionBehavior.allPanes;
    this.allowSplitterDock = false;
    this.layout = undefined;
    this.draggedPane = undefined;
    this.dropPosition = undefined;
    this.activePane = null;
  }
  flyoutPaneChanged(newValue) {
    if (newValue) {
      this.scheduleAfterRender(() => {
        this.setFocus(newValue);
      });
    }
  }
  containedInBoundariesChanged() {
    if (this.loaded) {
      this.restrictFloatingPaneSize();
    }
  }
  /**
   * Gets the direction of the Dock Manager.
   */
  get direction() {
    return Utils.getDirection(this.element);
  }
  layoutChanged() {
    this.service.processLayout();
    if (!this.isDragging) {
      this.layoutChange.emit();
    }
  }
  draggedPaneChanged() {
    if (this.draggedPane) {
      const panes = this.draggedPane.type === IgcDockManagerPaneType.contentPane ?
        [this.draggedPane] :
        this.service.getChildContentPanes(this.draggedPane);
      this.showDockingIndicators = panes.every(p => p.allowDocking !== false) && !this.anyPaneIgnoringResizeConstraints;
      this.documentOnlyDrag = panes.some(p => p.documentOnly);
    }
    else {
      this.showDockingIndicators = false;
      this.documentOnlyDrag = false;
    }
  }
  dropPositionChanged() {
    var _a, _b;
    this.paneDragMoved((_a = this.dropPosition) === null || _a === void 0 ? void 0 : _a.x, (_b = this.dropPosition) === null || _b === void 0 ? void 0 : _b.y);
  }
  activePaneChange(newValue, oldValue) {
    var _a;
    const args = { newPane: newValue, oldPane: oldValue };
    if (this.flyoutPane && newValue && newValue !== this.flyoutPane) {
      this.service.flyoutPane(this.flyoutPane);
    }
    this.activePaneChanged.emit(args);
    this.contentPaneId = (newValue === null || newValue === void 0 ? void 0 : newValue.contentId) || this.contentPaneId;
    if (this.activePaneInternalSet) {
      this.activePaneInternalSet = false;
      return;
    }
    if (newValue) {
      if (newValue.isPinned === false) {
        requestAnimationFrame(() => {
          this.service.flyoutPane(newValue);
        });
        // return because setFocus will be called in flyoutPaneChanged
        this.paneToFocus = newValue;
        return;
      }
      else {
        const parent = this.service.getParent(newValue);
        if (parent.type === IgcDockManagerPaneType.tabGroupPane && parent.panes.length > 1) {
          const visibleTabs = this.service.getVisibleContentPanes(parent);
          const tabHeaderElement = this.tabHeadersMap.get(newValue);
          const firstTabRect = this.tabHeadersMap
            .get(visibleTabs[0])
            .getBoundingClientRect();
          const headerRect = tabHeaderElement.getBoundingClientRect();
          // return if the selected index is changed because setFocus will be called in handleTabSelectedChanged
          if (headerRect.top !== firstTabRect.top) { // header is hidden
            this.service.selectHiddenTab(parent, newValue);
            this.paneToFocus = newValue;
            return;
          }
          else {
            const visibleIndex = visibleTabs.indexOf(newValue);
            const selectedIndex = (_a = parent.selectedIndex) !== null && _a !== void 0 ? _a : 0;
            if (selectedIndex !== visibleIndex) {
              parent.selectedIndex = visibleIndex;
              this.paneToFocus = newValue;
              return;
            }
          }
        }
      }
    }
    this.scheduleAfterRender(() => {
      this.setFocus(newValue);
    });
  }
  handleFocusOut(_event) {
    if (this.paneToFocus) {
      return;
    }
    const rootNode = this.getRootNode();
    if (!this._tabHeaderIconClicked && !this.isDragging) {
      requestAnimationFrame(() => {
        if (!this.element.contains(rootNode.activeElement)) {
          this.clearActivePane();
        }
      });
    }
    this._tabHeaderIconClicked = false;
  }
  handleKeyDown(event) {
    this.keyboardService.handleKeydown(event);
  }
  handlePointerDown(event) {
    const pathHTMLElements = event.composedPath().filter(e => e instanceof HTMLElement).map(e => e);
    const isMaximizeOrMinimize = pathHTMLElements.filter(e => e.attributes.getNamedItem('part') &&
      (e.attributes.getNamedItem('part').value === 'tabs-minimize-button' || e.attributes.getNamedItem('part').value === 'tabs-maximize-button')).length > 0;
    if (isMaximizeOrMinimize) {
      this.reason = ActionReason.maximizeOrMinimize;
    }
  }
  handlePointerUp() {
    this.reason = null;
  }
  async dropPane() {
    const docked = this.handlePaneDragEnd();
    return Promise.resolve(docked);
  }
  async removePane(pane) {
    this.service.removePane(pane);
  }
  async focusPane(contentId) {
    const contentPane = this.service.getContent(contentId);
    const parentPane = this.service.getRootParent(contentPane);
    const isFloating = this.service.isFloatingPane(parentPane);
    const isUnpinned = contentPane.isPinned !== undefined && !contentPane.isPinned;
    if (isFloating) {
      this.service.bringFloatingPaneOnTop(parentPane);
      this.activePane = contentPane;
    }
    else if (isUnpinned) {
      this.handlePaneFlyout(contentPane);
    }
    else {
      this.activePane = contentPane;
    }
  }
  connectedCallback() {
    this.element.shadowRoot.addEventListener('focusout', this.handleShadowRootFocusOut);
    this.domObserver.observe(this.element, { attributes: true, subtree: true, childList: true });
    const dockManagerChildren = Array.from(this.element.children);
    for (const htmlElem of dockManagerChildren) {
      const elem = htmlElem;
      elem === null || elem === void 0 ? void 0 : elem.addEventListener('focusin', this.handleCustomContentFocusIn);
    }
  }
  componentWillLoad() {
    this.contextMenuMeta = null;
    this.navigationPaneMeta = null;
    if (this.layout) {
      this.updateLayout();
      this.layoutChanged();
    }
    if (!this.resourceStrings) {
      const closestElement = this.element.closest('[lang]');
      const lang = closestElement ? closestElement.lang.toLowerCase() : 'en';
      this.resourceStrings = resourceStringsMap.has(lang) ? resourceStringsMap.get(lang) : resourceStringsMap.get('en');
    }
  }
  componentWillRender() {
    const lastMaximizedPane = this.service.getLastMaximizedPane();
    if (lastMaximizedPane && this.maximizedPane !== lastMaximizedPane) {
      const shouldFlyout = 'isPinned' in lastMaximizedPane && !lastMaximizedPane.isPinned;
      if (shouldFlyout) {
        this.handlePaneFlyout(lastMaximizedPane);
      }
      this.handleMaximize(lastMaximizedPane);
    }
  }
  componentDidRender() {
    while (this.scheduledCallbacks.length) {
      const callback = this.scheduledCallbacks.shift();
      callback();
    }
    if (this.service.keyboardDockPane) {
      this.setActivePane(this.service.keyboardDockPane);
      this.setFocus(this.service.keyboardDockPane);
      this.service.keyboardDockPane = null;
    }
    this.processTemplates();
  }
  componentDidLoad() {
    this.splitterDockingIndicator = this.element.shadowRoot.querySelector('igc-splitter-docking-indicator-component');
    this.loaded = true;
    this.resizeObserver.observe(this.element);
  }
  disconnectedCallback() {
    this.element.shadowRoot.removeEventListener('focusout', this.handleShadowRootFocusOut);
    this.removeCustomContentEventListeners();
    this.domObserver.disconnect();
    this.resizeObserver.disconnect();
  }
  focusElement() {
    this.element.focus();
  }
  updateLayout() {
    //Check for useFixedSize in the layout and create a placeholder pane for resizing.
    this.service.checkAndAddPlaceholderPane(this.layout.rootPane);
    if (!this.layout.floatingPanes)
      return;
    for (const p of this.layout.floatingPanes) {
      // Do not add a placeholder pane for floating pane's root splitPane
      p.panes.forEach(cp => this.service.checkAndAddPlaceholderPane(cp));
      // Should we add a resolved prop instead of setting useFixedSize to false?
      // Used to prevent the root parent splitPane of having flex grow and shrink set to 0 which breaks floating pane resizing.
      p.useFixedSize = false;
    }
  }
  handleTabsRendered(pane) {
    var _a;
    if (this.reason !== ActionReason.drop) {
      return;
    }
    if (((_a = pane.panes) === null || _a === void 0 ? void 0 : _a.length) > 0) {
      const focusablePane = this.findFocusablePane(pane.panes);
      if ((focusablePane === null || focusablePane === void 0 ? void 0 : focusablePane.contentId) === this.contentPaneId) {
        pane.selectedIndex = pane.panes.indexOf(focusablePane);
        this.setActivePane(focusablePane);
        this.setFocus(focusablePane);
      }
    }
  }
  handleSplitPaneRendered(panes) {
    if (this.reason !== ActionReason.drop) {
      return;
    }
    const focusablePane = this.findFocusablePane(panes);
    if ((focusablePane === null || focusablePane === void 0 ? void 0 : focusablePane.contentId) === this.contentPaneId) {
      this.setActivePane(focusablePane);
      this.setFocus(focusablePane);
    }
  }
  findFocusablePane(panes, candidate) {
    const splitPanes = panes.filter(p => p.type === IgcDockManagerPaneType.splitPane);
    for (const pane of splitPanes) {
      if (candidate) {
        return candidate;
      }
      const splitPane = pane;
      candidate = this.findFocusablePane(splitPane.panes);
    }
    const tabGroupPanes = panes.filter(p => p.type === IgcDockManagerPaneType.tabGroupPane);
    for (const pane of tabGroupPanes) {
      if (candidate) {
        return candidate;
      }
      const tabGroupPane = pane;
      candidate = tabGroupPane.selectedIndex ? tabGroupPane.panes[tabGroupPane.selectedIndex] : this.findFocusablePane(tabGroupPane.panes);
    }
    const docHost = panes.filter(p => p.type === IgcDockManagerPaneType.documentHost);
    for (const pane of docHost) {
      if (candidate) {
        return candidate;
      }
      const docHostPane = pane;
      candidate = this.findFocusablePane([docHostPane.rootPane]);
    }
    candidate = panes.find(p => p.type === IgcDockManagerPaneType.contentPane
      && p.contentId === this.contentPaneId) || candidate;
    if (candidate) {
      return candidate;
    }
    return panes.find(p => p.type === IgcDockManagerPaneType.contentPane);
  }
  processTemplates() {
    if (this.templates.size === 0) {
      return;
    }
    this.templatableComponents.forEach(component => {
      const template = this.templates.get(component.slot);
      if (template) {
        template.setAttribute('slot', component.targetSlot);
        template.setAttribute('part', component.targetPart);
        const componentMatches = this.element.shadowRoot.querySelectorAll(component.targetName);
        componentMatches.forEach(match => {
          const slottedClose = match.querySelectorAll('[slot="' + component.targetSlot + '"]');
          if (slottedClose.length === 0) {
            match.appendChild(template.cloneNode(true));
          }
        });
      }
    });
  }
  getRootNode() {
    return this.element.getRootNode();
  }
  scheduleAfterRender(callback) {
    this.scheduledCallbacks.push(callback);
  }
  handleTabHeaderFocus(pane) {
    if (!this._tabHeaderIconClicked) {
      this.setActivePane(pane);
    }
  }
  handleContentPaneFocus(pane) {
    this.setActivePane(pane);
  }
  handleUnpinnedHeaderFocus(pane) {
    // delay the setting of the active pane, in case the unpinned tab is clicked it should focus the content pane first
    requestAnimationFrame(() => {
      this.setActivePane(pane);
    });
  }
  setFocus(pane) {
    if (!pane) {
      return;
    }
    this.paneToFocus = null;
    let elemToFocus = this.contentPanesElementMap.get(pane);
    if (!elemToFocus) {
      elemToFocus = this.tabHeadersMap.get(pane);
    }
    if (!elemToFocus) {
      elemToFocus = this.unpinnedHeadersMap.get(pane);
    }
    if (!elemToFocus) {
      throw new Error('No matching pane!');
    }
    elemToFocus.focus();
  }
  setActivePane(pane) {
    if (this.activePane !== pane) {
      this.activePaneInternalSet = true;
      this.activePane = pane;
    }
  }
  removeCustomContentEventListeners() {
    const dockManagerChildren = Array.from(this.element.children);
    dockManagerChildren.forEach(child => {
      child.removeEventListener('focusin', this.handleCustomContentFocusIn);
    });
  }
  mutationCallback(records) {
    records.filter(rec => rec.type === 'childList').forEach(child => {
      child.addedNodes.forEach(c => {
        c.addEventListener('focusin', this.handleCustomContentFocusIn);
      });
      child.removedNodes.forEach(r => {
        r.removeEventListener('focusin', this.handleCustomContentFocusIn);
      });
    });
  }
  get isValidDrop() {
    return this._isValidDrop;
  }
  set isValidDrop(value) {
    if (this._isValidDrop !== value) {
      this._isValidDrop = value;
      document.body.style.cursor = this._isValidDrop ? 'default' : 'not-allowed';
    }
  }
  restrictFloatingPaneSize() {
    if (this.containedInBoundaries) {
      const dockManagerRect = this.element.getBoundingClientRect();
      const floatingPanes = Array.from(this.element.shadowRoot.querySelectorAll('igc-floating-pane-component'));
      floatingPanes.forEach(floatingPane => {
        const paneElement = floatingPane.querySelector('igc-split-pane-component');
        const pane = this.panesElementMap.getByValue(paneElement);
        if (floatingPane.getBoundingClientRect().height > dockManagerRect.height) {
          floatingPane.floatingHeight = pane.floatingHeight = dockManagerRect.height;
          floatingPane.style.height = floatingPane.floatingHeight + 'px';
        }
        if (floatingPane.getBoundingClientRect().bottom > dockManagerRect.bottom) {
          floatingPane.floatingLocation.y = pane.floatingLocation.y = dockManagerRect.bottom - floatingPane.getBoundingClientRect().height;
          floatingPane.style.top = floatingPane.floatingLocation.y + 'px';
        }
        if (floatingPane.getBoundingClientRect().y < dockManagerRect.y) {
          floatingPane.floatingLocation.y = dockManagerRect.y;
          floatingPane.style.top = floatingPane.floatingLocation.y + 'px';
        }
        if (floatingPane.getBoundingClientRect().width > dockManagerRect.width) {
          floatingPane.floatingWidth = dockManagerRect.width;
          floatingPane.style.width = floatingPane.floatingWidth + 'px';
        }
        if (floatingPane.getBoundingClientRect().right > dockManagerRect.right) {
          floatingPane.floatingLocation.x = pane.floatingLocation.x = dockManagerRect.right - floatingPane.getBoundingClientRect().width;
          floatingPane.style.left = floatingPane.floatingLocation.x + 'px';
        }
        if (floatingPane.getBoundingClientRect().left < dockManagerRect.left) {
          floatingPane.floatingLocation.x = dockManagerRect.x;
          floatingPane.style.left = floatingPane.floatingLocation.x + 'px';
        }
      });
    }
  }
  isPaneSizeWithinConstraints(pane, offset, orientation, position = 'next') {
    let size, minSize, maxSize;
    if (orientation === IgcSplitPaneOrientation.horizontal) {
      size = pane.getBoundingClientRect().width;
      minSize = parseFloat(getComputedStyle(pane).minWidth);
      maxSize = parseFloat(getComputedStyle(pane).maxWidth);
    }
    else {
      size = pane.getBoundingClientRect().height;
      minSize = parseFloat(getComputedStyle(pane).minHeight);
      maxSize = parseFloat(getComputedStyle(pane).maxHeight);
    }
    if (position === 'previous') {
      return offset < 0 ? ((size + offset) >= minSize || isNaN(minSize) || minSize === 0) : ((size + offset) <= maxSize || isNaN(maxSize));
    }
    return offset > 0 ? ((size - offset) >= minSize || isNaN(minSize)) : ((size - offset) <= maxSize || isNaN(maxSize));
  }
  handleFlyoutSplitterResizeEnd(pane, orientation, event) {
    const splitter = event.target;
    const previousPane = splitter.previousElementSibling;
    const minSize = orientation === IgcSplitPaneOrientation.horizontal
      ? parseFloat(getComputedStyle(previousPane).minWidth)
      : parseFloat(getComputedStyle(previousPane).minHeight);
    if (!isNaN(minSize) && (minSize > pane.unpinnedSize || minSize > IGC_DEFAULT_UNPIN_PANE_SIZE)) {
      pane.unpinnedSize = minSize;
    }
    // If new pane size does not meet the size constraints, do not resize
    let parent = this.service.getParent(pane);
    let offset = this.service.isSplitPaneIgnoringResizeConstraints(parent) ||
      this.isPaneSizeWithinConstraints(previousPane, event.detail, orientation, 'previous') ?
      this.splitterOffset * event.detail :
      0;
    this.service.resizeFlyoutPane(offset);
    let rect;
    rect = splitter.previousElementSibling.getBoundingClientRect();
    const paneWidth = orientation === IgcSplitPaneOrientation.horizontal ? rect.width + offset : rect.width;
    const paneHeight = orientation === IgcSplitPaneOrientation.vertical ? rect.height + offset : rect.height;
    this.splitterResizeEnd.emit({ pane, orientation, paneWidth, paneHeight });
  }
  handleSplitterResizeStart(pane, event) {
    const splitter = event.target;
    const orientation = splitter.splitPaneOrientation;
    let rect;
    const splitterEl = event.target;
    rect = pane === this.flyoutPane ? splitterEl.previousElementSibling.getBoundingClientRect() : splitterEl.nextElementSibling.getBoundingClientRect();
    const paneWidth = rect.width;
    const paneHeight = rect.height;
    const resizeStartEvent = this.splitterResizeStart.emit({ pane, orientation, paneWidth, paneHeight });
    if (resizeStartEvent.defaultPrevented) {
      this.splitterOffset = 0;
      splitter.showDragGhost = false;
    }
    else {
      this.splitterOffset = 1;
      splitter.showDragGhost = true;
    }
  }
  handleSplitterResizeEnd(parentPane, pane, event) {
    const splitter = event.target;
    const previousPane = splitter.previousElementSibling;
    const nextPane = splitter.nextElementSibling;
    const splitPane = splitter.parentElement;
    const splitPaneChildren = Array.from(splitPane.children);
    const paneComponents = splitPaneChildren.filter(c => c.tagName.toLowerCase() !== 'igc-splitter-component');
    const sizeProperty = parentPane.orientation === IgcSplitPaneOrientation.horizontal ? 'offsetWidth' : 'offsetHeight';
    const sizeSum = paneComponents.reduce((s, p) => p[sizeProperty] + s, 0);
    // If the new size is smaller or bigger that the size constraints, do not resize
    const offset = parentPane.useFixedSize ||
      this.isPaneSizeWithinConstraints(previousPane, event.detail, parentPane.orientation, 'previous') &&
        this.isPaneSizeWithinConstraints(nextPane, event.detail, parentPane.orientation) ? event.detail : 0;
    let offsetPercentage = this.splitterOffset * offset / sizeSum;
    let rtl = false;
    if (this.element.dir !== '') {
      rtl = this.element.dir === 'rtl';
    }
    else {
      let parent = this.element.parentElement;
      while (parent) {
        if (parent.dir !== '') {
          rtl = parent.dir === 'rtl';
          break;
        }
        parent = parent.parentElement;
      }
    }
    if (rtl && parentPane.orientation === IgcSplitPaneOrientation.horizontal) {
      offsetPercentage *= -1;
    }
    this.service.resizePane(pane, offsetPercentage);
    const orientation = parentPane.orientation;
    let rect;
    rect = splitter.nextElementSibling.getBoundingClientRect();
    const paneWidth = orientation === IgcSplitPaneOrientation.horizontal ? rect.width - offsetPercentage * sizeSum : rect.width;
    const paneHeight = orientation === IgcSplitPaneOrientation.vertical ? rect.height - offsetPercentage * sizeSum : rect.height;
    this.splitterResizeEnd.emit({ pane, orientation, paneWidth, paneHeight });
  }
  clearActivePane() {
    if (this.activePane && this.shouldClearActivePane && this.reason !== ActionReason.drop) {
      this.setActivePane(null);
    }
  }
  handlePinToggle(pane) {
    const isPinned = this.service.getActualIsPinned(pane);
    const unpinBehavior = !isPinned ? IgcPaneActionBehavior.selectedPane : this.unpinBehavior;
    this.shouldClearActivePane = false;
    this.service.togglePin(pane, unpinBehavior);
    this.scheduleAfterRender(() => {
      this.shouldClearActivePane = true;
      this.setFocus(pane);
    });
  }
  handlePaneClose(pane) {
    this.focusElement();
    this.service.closePane(pane);
  }
  handleFloatingPaneClose(pane) {
    this.focusElement();
    this.service.closeFloatingPane(pane);
  }
  handleMaximize(pane) {
    this.service.maximizePane(pane);
    this.scheduleAfterRender(() => {
      const paneToFocus = this.findFocusablePane([pane]);
      this.setFocus(paneToFocus);
    });
  }
  handleUnpinnedTabMouseDown(pane, event) {
    const target = event.currentTarget;
    const rootNode = this.getRootNode();
    this.contentPaneId = pane.id;
    requestAnimationFrame(() => {
      if (this.element === rootNode.activeElement || !this.element.contains(rootNode.activeElement)) {
        target.focus();
      }
      this.handlePaneFlyout(pane);
    });
  }
  handlePaneFlyout(pane) {
    this.service.flyoutPane(pane);
  }
  handleUnpinnedTabKeyDown(pane, event) {
    if (event.key === 'Enter' || event.key === ' ') {
      this.handlePaneFlyout(pane);
    }
  }
  handlePaneDragStart(pane, event) {
    let rect;
    if (pane.type === IgcDockManagerPaneType.contentPane) {
      const header = event.target;
      const parent = this.service.getParent(pane);
      rect = (pane !== this.flyoutPane && parent.type === IgcDockManagerPaneType.tabGroupPane) ?
        header.closest('igc-tabs-component').getBoundingClientRect() :
        header.parentElement.getBoundingClientRect();
      this.contentPaneId = pane.contentId;
    }
    this.shouldClearActivePane = false;
    this.focusElement();
    const dragStarted = this.service.dragPaneStart(pane, rect, event.detail.clientX, event.detail.clientY, this.anyPaneIgnoringResizeConstraints);
    this.isDragging = true;
    this.scheduleAfterRender(() => {
      this.shouldClearActivePane = true;
      if (pane.type === IgcDockManagerPaneType.contentPane) {
        this.contentPaneId = pane.contentId;
        return;
      }
      if (pane.type === IgcDockManagerPaneType.splitPane) {
        const targetPane = this.findFocusablePane(pane.panes);
        if (targetPane) {
          this.contentPaneId = targetPane.contentId;
          this.setActivePane(targetPane);
        }
      }
    });
    if (!dragStarted) {
      this.isDragging = false;
      event.detail.cancel = true;
    }
    else {
      this.service.dockManagerRect = this.element.getBoundingClientRect();
    }
  }
  handlePaneDragMove(event) {
    this.paneDragMoved(event.detail.clientX, event.detail.clientY, event);
  }
  paneDragMoved(clientX, clientY, event) {
    this.handleDropPositionChange(clientX, clientY, event === null || event === void 0 ? void 0 : event.target);
    const x = event === null || event === void 0 ? void 0 : event.detail.clientX;
    const y = event === null || event === void 0 ? void 0 : event.detail.clientY;
    this.cursorLocation = { x, y };
    if (this.proximityDock) {
      this.service.proximityDockPosition = this.getProximityDockPosition();
      this.service.dropTargetParentRect = this.getInnerDropTargetParentRect();
    }
    const dragMoved = this.service.dragPaneMove(x, y);
    if (event && !dragMoved) {
      event.detail.cancel = true;
    }
    // After successfull pane drag, check if there is a splitPane with useFixedSize = true
    // that is left with a single placeholder pane and remove it.
    // This would happen only when the last non placeholder pane of a split pane gets dragged.
    if (dragMoved) {
      const elements = this.element.shadowRoot.elementsFromPoint(clientX, clientY);
      const splitPanes = elements.filter(e => e.tagName.toLowerCase() === 'igc-split-pane-component');
      for (const sp of splitPanes) {
        const splitPane = this.panesElementMap.getByValue(sp);
        if (splitPane.useFixedSize &&
          splitPane.panes.length === 1 &&
          splitPane.panes[0].type === IgcDockManagerPaneType.contentPane &&
          splitPane.panes[0].contentId === "placeholder") {
          this.service.closePane(splitPane.panes[0]);
        }
      }
    }
  }
  getInnerDropTargetParentRect() {
    if (this.dropTargetPaneInfo && this.service.proximityDockPosition) {
      const targetDocHostParent = this.service.getParent(this.dropTargetPaneInfo.docHost);
      // In proximityDock, targetDocHostParent can be undefined when elementsFromPoint returns an empty splitPane meaning the dragged pane is over it.
      const isIndicatorVertical = targetDocHostParent && Utils.isDockingIndicatorVertical(this.service.proximityDockPosition);
      const isSplitPaneVertical = targetDocHostParent && Utils.isSplitPaneVertical(targetDocHostParent);
      const isSameSplitPane = ((isIndicatorVertical && isSplitPaneVertical) || (!isIndicatorVertical && !isSplitPaneVertical));
      // If its over splitter or if docHost without splitter -> outer dock
      // For top/bottom outer dock we need the docHost's parent rect
      // For left/right outer dock we need the docHost's rootPane rect
      // If its not over splitter -> innerDock
      const targetPaneParent = this.allowOuterProximityDock && Utils.isDockingIndicatorOuter(this.service.proximityDockPosition)
        ? isSameSplitPane
          ? targetDocHostParent
          : this.dropTargetPaneInfo.docHost.rootPane
        : this.service.getParent(this.dropTargetPaneInfo.pane);
      const targetPaneParentRect = this.panesElementMap.has(targetPaneParent) ? this.panesElementMap.get(targetPaneParent).getBoundingClientRect() : null;
      return targetPaneParentRect;
    }
  }
  getProximityDockPosition() {
    const dropTarget = this.dropTargetPaneInfo;
    if (!dropTarget) {
      return null;
    }
    const pane = dropTarget.pane;
    const targetPaneRect = dropTarget.targetRect;
    const centerX = (targetPaneRect.left + targetPaneRect.right) / 2;
    const centerY = (targetPaneRect.top + targetPaneRect.bottom) / 2;
    const isRTL = this.direction === 'rtl';
    const targetPane = pane.type === IgcDockManagerPaneType.tabGroupPane && pane.selectedIndex > -1
      ? pane.panes[pane.selectedIndex]
      : pane;
    const acceptsInnerDock = targetPane.type === IgcDockManagerPaneType.contentPane && targetPane.acceptsInnerDock !== undefined && this.allowInnerDock
      ? targetPane.acceptsInnerDock
      : this.allowInnerDock;
    const isOverSplitter = this.splitterRect
      ? this.cursorLocation.x >= this.splitterRect.left
        && this.cursorLocation.x <= this.splitterRect.right
        && this.cursorLocation.y >= this.splitterRect.top
        && this.cursorLocation.y <= this.splitterRect.bottom
      : false;
    // If outer proximity dock is enabled, check if there is splitter and if mouse is within splitter boundaries.
    if (this.allowOuterProximityDock && this.splitterRect) {
      if (this.cursorLocation.x >= this.splitterRect.left - this.proximityOuterDockThreshold
        && this.cursorLocation.x <= this.splitterRect.left) {
        return IgcDockingIndicatorPosition.outerLeft;
      }
      else if (this.cursorLocation.x <= this.splitterRect.right + this.proximityOuterDockThreshold
        && this.cursorLocation.x >= this.splitterRect.right) {
        return IgcDockingIndicatorPosition.outerRight;
      }
    }
    const allowOuterTopDock = this.allowOuterProximityDock && this.cursorLocation.y < targetPaneRect.top + this.proximityOuterDockThreshold;
    const allowOuterBottomDock = this.allowOuterProximityDock && this.cursorLocation.y > targetPaneRect.bottom - this.proximityOuterDockThreshold;
    const allowOuterLeftDock = this.allowOuterProximityDock && this.cursorLocation.x < targetPaneRect.left + this.proximityOuterDockThreshold;
    const allowOuterRightDock = this.allowOuterProximityDock && this.cursorLocation.x > targetPaneRect.right - this.proximityOuterDockThreshold;
    if (this.cursorLocation.y > targetPaneRect.bottom - this.proximityDockThreshold
      && this.cursorLocation.x - targetPaneRect.left > targetPaneRect.bottom - this.cursorLocation.y
      && targetPaneRect.right - this.cursorLocation.x > targetPaneRect.bottom - this.cursorLocation.y
      && !isOverSplitter) {
      return allowOuterBottomDock ? IgcDockingIndicatorPosition.outerBottom : IgcDockingIndicatorPosition.bottom;
    }
    else if (this.cursorLocation.y < targetPaneRect.top + this.proximityDockThreshold
      && this.cursorLocation.x - targetPaneRect.left > this.cursorLocation.y - targetPaneRect.top
      && targetPaneRect.right - this.cursorLocation.x > this.cursorLocation.y - targetPaneRect.top
      && !isOverSplitter) {
      return allowOuterTopDock ? IgcDockingIndicatorPosition.outerTop : IgcDockingIndicatorPosition.top;
    }
    else if (acceptsInnerDock
      && this.cursorLocation.x > centerX - this.proximityDockThreshold
      && this.cursorLocation.x < centerX + this.proximityDockThreshold
      && this.cursorLocation.y > centerY - this.proximityDockThreshold
      && this.cursorLocation.y < centerY + this.proximityDockThreshold
      && !isOverSplitter) {
      return IgcDockingIndicatorPosition.center;
    }
    else if (!isRTL && this.cursorLocation.x < targetPaneRect.left + this.proximityDockThreshold
      && this.cursorLocation.y - targetPaneRect.top > this.cursorLocation.x - targetPaneRect.left
      && targetPaneRect.bottom - this.cursorLocation.y > this.cursorLocation.x - targetPaneRect.left) {
      return allowOuterLeftDock ? IgcDockingIndicatorPosition.outerLeft : IgcDockingIndicatorPosition.left;
    }
    else if (!isRTL && this.cursorLocation.x > targetPaneRect.right - this.proximityDockThreshold
      && this.cursorLocation.y - targetPaneRect.top > targetPaneRect.right - this.cursorLocation.x
      && targetPaneRect.bottom - this.cursorLocation.y > targetPaneRect.right - this.cursorLocation.x) {
      return allowOuterRightDock ? IgcDockingIndicatorPosition.outerRight : IgcDockingIndicatorPosition.right;
    }
    else if (isRTL && this.cursorLocation.x > targetPaneRect.right - this.proximityDockThreshold
      && targetPaneRect.bottom - this.cursorLocation.y > this.cursorLocation.x - targetPaneRect.right
      && this.cursorLocation.y - targetPaneRect.top > this.cursorLocation.x - targetPaneRect.right) {
      return allowOuterRightDock ? IgcDockingIndicatorPosition.outerRight : IgcDockingIndicatorPosition.right;
    }
    else if (isRTL && this.cursorLocation.x < targetPaneRect.left + this.proximityDockThreshold
      && targetPaneRect.bottom - this.cursorLocation.y > targetPaneRect.left - this.cursorLocation.x
      && this.cursorLocation.y - targetPaneRect.top > targetPaneRect.left - this.cursorLocation.x) {
      return allowOuterLeftDock ? IgcDockingIndicatorPosition.outerLeft : IgcDockingIndicatorPosition.left;
    }
    return null;
  }
  handlePaneDragEnd() {
    this.setSplitterDockingIndicatorVisibility('hidden');
    const docked = this.service.dragPaneEnd();
    this.isDragging = false;
    if (docked) {
      const activePane = this.activePane;
      this.shouldClearActivePane = false;
      this.reason = ActionReason.drop;
      this.focusElement();
      if (activePane) {
        this.scheduleAfterRender(() => {
          this.setActivePane(activePane);
        });
      }
      this.scheduleAfterRender(() => {
        this.shouldClearActivePane = true;
      });
    }
    if (this.activePane) {
      this.setFocus(this.activePane);
    }
    return docked;
  }
  handleTabHeaderDragStart(pane, event) {
    this.shouldMovePane = false;
    const path = this.service.getPanePath(pane);
    const rootParent = path[0];
    if (rootParent !== this.layout.rootPane &&
      path.some(p => p.type === IgcDockManagerPaneType.documentHost)) {
      const childPanes = this.service.getChildContentPanes(rootParent);
      if (childPanes.length === 1) {
        // pane is a single document host tab in a floating pane
        this.shouldMovePane = true;
        return this.handlePaneDragStart(rootParent, event);
      }
    }
    this.shouldClearActivePane = false;
    this.focusElement();
    const dragStarted = this.service.dragTabStart(pane);
    this.isDragging = true;
    this.scheduleAfterRender(() => {
      this.shouldClearActivePane = true;
    });
    if (!dragStarted) {
      this.isDragging = false;
      event.detail.cancel = true;
    }
  }
  handleTabHeaderDragMove(pane, event) {
    if (this.shouldMovePane) {
      return this.handlePaneDragMove(event);
    }
    const header = event.target;
    const headerRect = header.getBoundingClientRect();
    const tabs = header.closest('igc-tabs-component');
    const tabsRect = tabs.getBoundingClientRect();
    const tabHeaders = Array.from(tabs.querySelectorAll('igc-tab-header-component'));
    const headerIndex = tabHeaders.indexOf(header);
    const prevHeaderRect = headerIndex > 0 ? tabHeaders[headerIndex - 1].getBoundingClientRect() : null;
    const nextHeaderRect = headerIndex < tabHeaders.length - 1 ? tabHeaders[headerIndex + 1].getBoundingClientRect() : null;
    const lastVisibleHeaderRect = tabHeaders.filter(th => th.getBoundingClientRect().top === headerRect.top).slice(-1)[0].getBoundingClientRect();
    const tabRectsInfo = {
      headerRect,
      prevHeaderRect,
      nextHeaderRect,
      lastVisibleHeaderRect,
      tabsRect
    };
    this.handleDropPositionChange(event.detail.clientX, event.detail.clientY, event === null || event === void 0 ? void 0 : event.target);
    const tabMoved = this.service.dragTabMove(pane, event.detail, tabRectsInfo, this.anyPaneIgnoringResizeConstraints);
    if (event && !tabMoved) {
      event.detail.cancel = true;
    }
  }
  handleTabHeaderDragEnd(pane) {
    const docked = this.handlePaneDragEnd();
    if (!docked) {
      this.activePane = pane;
    }
  }
  handleFloatingPaneResizeStart(pane, event) {
    const args = event.detail;
    this.isDragging = true;
    const resizeStarted = this.service.resizeFloatingPaneStart(pane, args.resizerLocation);
    if (!resizeStarted) {
      this.isDragging = false;
      args.dragMoveArgs.cancel = true;
    }
  }
  handleFloatingPaneResizeMove(pane, event) {
    this.service.resizeFloatingPane(pane, event.detail);
  }
  handleFloatingPaneResizeEnd(pane, event) {
    this.isDragging = false;
    this.service.resizeFloatingPaneEnd(pane, event.detail.resizerLocation);
  }
  handleDropPositionChange(clientX, clientY, target) {
    var _a;
    if (this.highlightedJoystick) {
      this.highlightedJoystick.style.boxShadow = '';
    }
    const elements = clientX && clientY ?
      this.element.shadowRoot.elementsFromPoint(clientX, clientY) :
      null;
    this.splitterRect = null;
    if (this.proximityDock) {
      const docHost = elements ? elements.filter(e => e.tagName.toLowerCase() === 'igc-document-host-component') : [];
      if (docHost.length > 0) {
        const splitter = docHost[0].querySelector('igc-splitter-component');
        if (splitter) {
          const splitterRect = splitter.getBoundingClientRect();
          this.splitterRect = splitterRect;
        }
        this.allowOuterProximityDock = true;
      }
      else {
        this.allowOuterProximityDock = false;
      }
    }
    if (!elements || !elements.length /*|| (isOverSplitter && this.splitterRect === null)*/) {
      this.service.dockingIndicator = null;
      this.dropTargetPaneInfo = null;
      return;
    }
    const topElement = elements[0];
    let indicatorTarget = null;
    let dockingIndicatorElement = topElement.closest('igc-root-docking-indicator-component');
    if (dockingIndicatorElement) {
      // root docking indicator
      this.service.dockingIndicator = {
        position: dockingIndicatorElement.position,
        isRoot: true,
        direction: this.direction,
      };
      this.highlightedJoystick = dockingIndicatorElement;
      this.highlightedJoystick.style.boxShadow = 'inset 0 0 0 3px rgba(0,0,0,0.2), inset 0 0 0 25px rgba(100,149,237,0.2)';
    }
    else {
      // regular docking indicator
      const joystickIcon = topElement.closest('igc-joystick-icon-component');
      if (joystickIcon && !joystickIcon.empty) {
        dockingIndicatorElement = joystickIcon;
        this.service.dockingIndicator = {
          position: joystickIcon.position,
          isRoot: false,
          direction: this.direction
        };
        const joystickIndicator = joystickIcon.closest('igc-joystick-indicator-component');
        indicatorTarget = joystickIndicator.dropTargetPaneInfo;
        this.highlightedJoystick = joystickIcon;
        this.highlightedJoystick.style.boxShadow = 'inset 0 0 0 3px rgba(0,0,0,0.2), inset 0 0 0 25px rgba(100,149,237,0.2)';
      }
    }
    if (dockingIndicatorElement &&
      (this.service.dockingIndicator.isRoot || this.dropTargetPaneInfo === indicatorTarget)) {
      this.service.dropTargetParentRect = this.getDropTargetParentRect();
      return;
    }
    this.service.dockingIndicator = null;
    if (target) {
      const draggedHeader = target;
      const draggedFloatingPane = draggedHeader.closest('igc-floating-pane-component');
      const topFloatingPane = elements.find(e => e.tagName.toLowerCase() === 'igc-floating-pane-component');
      if (topFloatingPane === draggedFloatingPane) {
        const index = elements.lastIndexOf(draggedFloatingPane);
        elements.splice(0, index + 1);
      }
    }
    let paneElement;
    for (const element of elements) {
      const tagName = element.tagName.toLowerCase();
      if (tagName === 'igc-content-pane-component' || tagName === 'igc-split-pane-component') {
        if (paneElement) {
          break;
        }
        paneElement = element;
        // Not sure why we dont break after finding split-pane first. This works with proximityDock so its temporarily here.
        // When we are over splitter and proximityDock is true, break after finding the first suitable splitPane.
        if (this.allowOuterProximityDock) {
          break;
        }
      }
      else if (tagName === 'igc-tabs-component' || tagName === 'igc-document-host-component') {
        paneElement = element;
        break;
      }
      else if (tagName === 'igc-floating-pane-component') {
        break;
      }
    }
    if (paneElement && this.panesElementMap.getByValue(paneElement)) {
      const dropPane = this.panesElementMap.getByValue(paneElement);
      if ((!this.dropTargetPaneInfo || this.dropTargetPaneInfo.pane !== dropPane) && dropPane !== this.draggedPane) {
        const docHost = this.service.getDocHostParent(dropPane);
        const dropPaneRoot = this.service.getRootParent(dropPane);
        const floatingPaneWithoutDocHost = this.layout.floatingPanes
          ? this.layout.floatingPanes.indexOf(dropPaneRoot) !== -1 &&
            !this.service.getChildDocHostRecursive(dropPaneRoot)
          : false;
        if (!this.documentOnlyDrag
          || docHost
          || floatingPaneWithoutDocHost
          || dropPane.type === IgcDockManagerPaneType.documentHost) {
          const paneRect = paneElement.getBoundingClientRect();
          if (dropPane.type === IgcDockManagerPaneType.contentPane) {
            this.paneVisibleRect = new DOMRect(paneRect.x, paneRect.y, paneRect.width, paneRect.height);
            this.paneVisibleRect = this.reduceRectBasedOnSplitters(this.paneVisibleRect, paneElement);
            this.paneVisibleRect = this.reduceRectBasedOnDockManagerBorders(this.paneVisibleRect);
          }
          this.dropTargetPaneInfo = {
            pane: dropPane,
            targetRect: paneRect,
            visibleTargetRect: (_a = this.paneVisibleRect) !== null && _a !== void 0 ? _a : paneRect,
            docHost,
            floatingPaneWithoutDocHost,
          };
          this.paneVisibleRect = null;
        }
        else {
          this.dropTargetPaneInfo = null;
        }
      }
    }
    else {
      this.dropTargetPaneInfo = null;
    }
    if (!this.allowSplitterDock)
      return;
    // if (clientX, clientY) is less than 20 pixels in either way to the nearest 'igc-splitter-component' element, show the splitterIndicator
    const splitters = this.element.shadowRoot.querySelectorAll('igc-splitter-component');
    let splitterFound = false;
    splitters.forEach(splitter => {
      if (splitterFound) {
        return;
      }
      let splitterRect = splitter.getBoundingClientRect();
      const horizontalDeviation = splitterRect.width < splitterRect.height ? 1 : 0;
      const verticalDeviation = splitterRect.width > splitterRect.height ? 1 : 0;
      const isOverSplitter = clientX >= splitterRect.left - (horizontalDeviation * 20)
        && clientX <= splitterRect.right + (horizontalDeviation * 20)
        && clientY >= splitterRect.top - (verticalDeviation * 20)
        && clientY <= splitterRect.bottom + (verticalDeviation * 20);
      const isSplitterHorizontal = splitterRect.width > splitterRect.height;
      if (isOverSplitter) {
        splitterRect = this.reduceRectBasedOnSplitters(splitterRect, splitter);
        splitterRect = this.reduceRectBasedOnDockManagerBorders(splitterRect);
        const isOverSplitterVisiblePart = clientX >= splitterRect.left - (horizontalDeviation * 20)
          && clientX <= splitterRect.right + (horizontalDeviation * 20)
          && clientY >= splitterRect.top - (verticalDeviation * 20)
          && clientY <= splitterRect.bottom + (verticalDeviation * 20);
        if (!isOverSplitterVisiblePart) {
          this.setSplitterDockingIndicatorVisibility('hidden');
        }
        const splitterParent = splitter.parentElement;
        const splitterParentIndex = elements.indexOf(splitterParent);
        const floatingPane = elements.slice().reverse().find(e => e.tagName.toLowerCase() === 'igc-floating-pane-component');
        const floatingPaneIndex = elements.indexOf(floatingPane);
        if (floatingPaneIndex > 0 && floatingPaneIndex < splitterParentIndex) {
          // there is a floating pane between the splitter area and the cursor
          return;
        }
        splitterFound = true;
        const splitterCenterX = splitterRect.left + splitterRect.width / 2;
        const splitterCenterY = splitterRect.top + splitterRect.height / 2;
        this.splitterDockingIndicator.style.left = `${splitterCenterX - this.splitterDockingIndicator.clientWidth / 2 - 1}px`;
        this.splitterDockingIndicator.style.top = `${splitterCenterY - this.splitterDockingIndicator.clientHeight / 2 - 1}px`;
        this.splitterDockingIndicator.position = isSplitterHorizontal ? IgcDockingIndicatorPosition.splitterHorizontal : IgcDockingIndicatorPosition.splitterVertical;
        this.setSplitterDockingIndicatorVisibility('visible');
        // check if cursor is over splitterIndicator
        if (!dockingIndicatorElement && this.splitterDockingIndicator.style.visibility === 'visible') {
          const splitterIndicatorRect = this.splitterDockingIndicator.getBoundingClientRect();
          const isOversplitterIndicator = clientX >= splitterIndicatorRect.left && clientX <= splitterIndicatorRect.right &&
            clientY >= splitterIndicatorRect.top && clientY <= splitterIndicatorRect.bottom;
          if (isOversplitterIndicator) {
            dockingIndicatorElement = this.splitterDockingIndicator;
            this.service.dockingIndicator = {
              position: isSplitterHorizontal ? IgcDockingIndicatorPosition.splitterHorizontal : IgcDockingIndicatorPosition.splitterVertical,
              isRoot: false,
              direction: this.direction
            };
            this.highlightedJoystick = this.splitterDockingIndicator;
            this.highlightedJoystick.style.boxShadow = 'inset 0 0 0 3px rgba(0,0,0,0.2), inset 0 0 0 25px rgba(100,149,237,0.2)';
            this.service.dropTargetParentRect = splitterRect;
            const paneToInsertAfter = this.panesElementMap.getByValue(splitter.previousElementSibling);
            this.service.splitterDockPane = paneToInsertAfter;
          }
        }
      }
      else {
        this.setSplitterDockingIndicatorVisibility('hidden');
      }
    });
  }
  reduceRectBasedOnSplitters(rect, element) {
    var _a, _b, _c, _d;
    let reducedRect = rect;
    let parentElement = element.parentElement;
    while (parentElement) {
      if (parentElement.previousElementSibling && ((_a = parentElement.previousElementSibling) === null || _a === void 0 ? void 0 : _a.tagName.toLowerCase()) === 'igc-splitter-component') {
        const parentPreviousElementSiblingRect = (_b = parentElement.previousElementSibling) === null || _b === void 0 ? void 0 : _b.getBoundingClientRect();
        reducedRect = this.getVisibleRect(reducedRect, parentPreviousElementSiblingRect, parentPreviousElementSiblingRect.width <= 5, true);
      }
      if (parentElement.nextElementSibling && ((_c = parentElement.nextElementSibling) === null || _c === void 0 ? void 0 : _c.tagName.toLowerCase()) === 'igc-splitter-component') {
        const parentNextElementSiblingRect = (_d = parentElement.nextElementSibling) === null || _d === void 0 ? void 0 : _d.getBoundingClientRect();
        reducedRect = this.getVisibleRect(reducedRect, parentNextElementSiblingRect, parentNextElementSiblingRect.width <= 5, false);
      }
      parentElement = parentElement.parentElement;
    }
    return reducedRect;
  }
  reduceRectBasedOnDockManagerBorders(rect) {
    let reducedRect = rect;
    const dockManagerRect = this.element.getBoundingClientRect();
    const dockManagerLeftRect = new DOMRect(dockManagerRect.x, dockManagerRect.y, 1, dockManagerRect.height);
    reducedRect = this.getVisibleRect(reducedRect, dockManagerLeftRect, true, true);
    const dockManagerRightRect = new DOMRect(dockManagerRect.right - 1, dockManagerRect.y, 1, dockManagerRect.height);
    reducedRect = this.getVisibleRect(reducedRect, dockManagerRightRect, true, false);
    const dockManagerTopRect = new DOMRect(dockManagerRect.x, dockManagerRect.y, dockManagerRect.width, 1);
    reducedRect = this.getVisibleRect(reducedRect, dockManagerTopRect, false, true);
    const dockManagerBottomRect = new DOMRect(dockManagerRect.x, dockManagerRect.bottom - 1, dockManagerRect.width, 1);
    reducedRect = this.getVisibleRect(reducedRect, dockManagerBottomRect, false, false);
    return reducedRect;
  }
  setSplitterDockingIndicatorVisibility(value) {
    if (this.splitterDockingIndicator && this.splitterDockingIndicator.style.visibility !== value) {
      this.splitterDockingIndicator.style.visibility = value;
      forceUpdate(this);
    }
  }
  getVisibleRect(rect, otherRect, isLeftOrRight, isLeftOrTop) {
    if (!rect || !otherRect || otherRect.width === 0 || otherRect.height === 0) {
      return rect;
    }
    let visibleRect = rect;
    if (isLeftOrRight) {
      if (isLeftOrTop && rect.left < otherRect.left && otherRect.right < rect.right) { // right side visible
        visibleRect.width = rect.right - otherRect.right;
        visibleRect.x = otherRect.right;
      }
      else if (!isLeftOrTop && rect.left < otherRect.left && otherRect.right < rect.right) { // left side visible
        visibleRect.width = otherRect.left - rect.x;
      }
    }
    else {
      if (isLeftOrTop && rect.top < otherRect.top && otherRect.bottom < rect.bottom) { // top side visible
        visibleRect.height = rect.bottom - otherRect.bottom;
        visibleRect.y = otherRect.bottom;
      }
      else if (!isLeftOrTop && rect.top < otherRect.top && otherRect.bottom < rect.bottom) { // bottom side visible
        visibleRect.height = otherRect.top - rect.y;
      }
    }
    // Ensure the visibleRect is within the bounds of the original rect
    if (visibleRect.width < 0 || visibleRect.height < 0) {
      return rect; // Element is not visible
    }
    return visibleRect;
  }
  getDropTargetParentRect() {
    const position = this.service.dockingIndicator.position;
    let rect;
    if (this.service.dockingIndicator.isRoot) {
      rect = this.dockedPanesContainer.getBoundingClientRect();
    }
    else {
      if (Utils.isDockingIndicatorOuter(position)) {
        const targetDocHostParent = this.service.getParent(this.dropTargetPaneInfo.docHost);
        const isIndicatorVertical = Utils.isDockingIndicatorVertical(position);
        const isSplitPaneVertical = Utils.isSplitPaneVertical(targetDocHostParent);
        const isSameSplitPane = ((isIndicatorVertical && isSplitPaneVertical) || (!isIndicatorVertical && !isSplitPaneVertical));
        const outerParent = isSameSplitPane ? targetDocHostParent : this.dropTargetPaneInfo.docHost.rootPane;
        const outerParentRect = this.panesElementMap.has(outerParent) ? this.panesElementMap.get(outerParent).getBoundingClientRect() : null;
        rect = outerParentRect;
      }
      else {
        const targetPaneParent = this.service.getParent(this.dropTargetPaneInfo.pane);
        const targetPaneParentRect = this.panesElementMap.has(targetPaneParent) ? this.panesElementMap.get(targetPaneParent).getBoundingClientRect() : null;
        rect = targetPaneParentRect;
      }
    }
    return rect;
  }
  handlePaneContentMouseDown(pane) {
    requestAnimationFrame(() => {
      const rootNode = this.getRootNode();
      if (this.element === rootNode.activeElement || !this.element.contains(rootNode.activeElement)) {
        this.setFocus(pane);
        this.contentPaneId = pane.contentId;
      }
    });
  }
  handlePaneContentScroll(pane, event) {
    const contentElement = event.target;
    const args = { pane, contentElement: contentElement };
    this.paneScroll.emit(args);
  }
  handlePaneHeaderMouseDown(pane) {
    const rootNode = this.getRootNode();
    requestAnimationFrame(() => {
      this.contentPaneId = pane.contentId;
      if (this.element === rootNode.activeElement || !this.element.contains(rootNode.activeElement)) {
        const paneElement = this.contentPanesElementMap.get(pane);
        paneElement === null || paneElement === void 0 ? void 0 : paneElement.focus();
      }
      else {
        this.setActivePane(pane);
      }
    });
  }
  handleTabHeaderMouseDown(pane, event) {
    const target = event.currentTarget;
    const rootNode = this.getRootNode();
    const pathHTMLElements = event.composedPath().filter(et => et instanceof HTMLElement).map(et => et);
    this._tabHeaderIconClicked = this.showHeaderIconOnHover !== 'none' &&
      pathHTMLElements.filter(el => el.attributes.getNamedItem('part') && el.attributes.getNamedItem('part').value === 'tab-header-more-options floating').length > 0;
    requestAnimationFrame(() => {
      this.contentPaneId = pane.contentId;
      this.reason = ActionReason.click;
      if (this.element === rootNode.activeElement || !this.element.contains(rootNode.activeElement)) {
        target.focus();
      }
      else {
        this.setActivePane(pane);
      }
    });
  }
  handleFloatingPaneMouseDown(pane, event) {
    const pathHTMLElements = event.composedPath().filter(e => e instanceof HTMLElement).map(e => e);
    const isContextMenu = pathHTMLElements.filter(e => e.attributes.getNamedItem('part') && e.attributes.getNamedItem('part').value === 'context-menu').length > 0;
    if (isContextMenu) {
      return;
    }
    this.service.bringFloatingPaneOnTop(pane);
    const isTabs = pathHTMLElements.filter(e => e.attributes.getNamedItem('part') && e.attributes.getNamedItem('part').value === 'tabs-container').length > 0;
    if (isTabs) {
      return;
    }
    const isFloatingPaneHeader = pathHTMLElements.filter(e => e.attributes.getNamedItem('part') && e.attributes.getNamedItem('part').value === 'pane-header floating window').length > 0;
    if (isFloatingPaneHeader) {
      let targetPane;
      let parent = this.service.getParent(this.activePane);
      while (parent) {
        if (parent === pane) {
          targetPane = this.activePane;
          break;
        }
        parent = this.service.getParent(parent);
      }
      requestAnimationFrame(() => {
        if (!targetPane) {
          targetPane = this.findFocusablePane(pane.panes);
        }
        if (!targetPane) {
          targetPane = pane.panes[0];
        }
        const nodeName = event.target.nodeName.toLowerCase();
        if (nodeName === 'igc-dockmanager') {
          this.setActivePane(targetPane);
          this.setFocus(targetPane);
        }
        this.contentPaneId = targetPane.contentId;
      });
    }
  }
  handleTabIconClick(p, position, isFloating, event) {
    if (this.showHeaderIconOnHover === 'none') {
      this.focusElement();
    }
    if (!isFloating && position === IgcTabHeadersPosition.bottom) {
      const elem = event.detail.target;
      const tabHeader = elem.closest('igc-tab-header-component');
      const moreOptionsContainer = tabHeader ?
        tabHeader.shadowRoot.querySelector('div[part*="tab-header-more-options"]') :
        elem.closest('div[part*="tab-header-more-options"]');
      this.contextMenuMeta = { target: moreOptionsContainer, menuItems: this.service.createContextMenuItems(p), position: this.contextMenuPosition };
    }
    else {
      this.contextMenuMeta = null;
      this.service.closeTabPane(p);
    }
  }
  handleContextMenuClosed() {
    this.contextMenuMeta = null;
  }
  handleTabSelectedIndexChanged(pane, event) {
    pane.selectedIndex = event.detail;
  }
  handleHiddenTabSelected(pane, event) {
    this.activePane = this.service.getVisibleContentPanes(pane)[event.detail];
  }
  handleSelectedTabOutOfView(pane, event) {
    const cp = this.service.getVisibleContentPanes(pane)[event.detail];
    this.service.selectHiddenTab(pane, cp);
  }
  handleTabIconKeyDown(iconName, p, ev) {
    if (iconName === 'arrow_drop_down' && ev.detail.key === 'ArrowDown') {
      const targetElem = ev.detail.target;
      const tabHeader = targetElem.closest('igc-tab-header-component');
      const moreOptionsContainer = tabHeader ?
        tabHeader.shadowRoot.querySelector('div[part="tab-header-more-options"]') :
        targetElem.closest('div[part="tab-header-more-options"]');
      this.contextMenuMeta = { target: moreOptionsContainer, menuItems: this.service.createContextMenuItems(p), position: this.contextMenuPosition };
    }
  }
  handleTabSelectedChanged(pane, ev) {
    if (ev.detail && pane === this.paneToFocus) {
      this.setFocus(pane);
    }
  }
  emitPaneHeaderConnected(pane, event) {
    this.paneHeaderConnected.emit({
      pane,
      element: event.detail
    });
  }
  emitPaneHeaderDisconnected(pane, event) {
    this.paneHeaderDisconnected.emit({
      pane,
      element: event.detail
    });
  }
  emitTabHeaderConnected(pane, event) {
    this.tabHeaderConnected.emit({
      pane,
      element: event.detail
    });
  }
  emitTabHeaderDisconnected(pane, event) {
    this.tabHeaderDisconnected.emit({
      pane,
      element: event.detail
    });
  }
  resolveAllowMaximize(pane) {
    return this.service.resolveAllowMaximize(pane);
  }
  removeHoveredPane() {
    this.hoveredPane = null;
  }
  togglePaneHeaderVisibility(pane, event) {
    const paneRect = this.contentPanesElementMap.get(pane).getBoundingClientRect();
    const withinPane = event.clientY >= paneRect.top && event.clientY < paneRect.top + PANE_HEADER_HEIGHT;
    if (withinPane) {
      this.hoveredPane = pane;
    }
    else {
      this.removeHoveredPane();
    }
  }
  renderButtonsTemplates() {
    return (h("template", null, h("slot", { name: "paneHeaderCloseButton", onSlotchange: this.updateTemplates }), h("slot", { name: "tabHeaderCloseButton", onSlotchange: this.updateTemplates }), h("slot", { name: "closeButton", onSlotchange: this.updateTemplates }), h("slot", { name: "moreTabsButton", onSlotchange: this.updateTemplates }), h("slot", { name: "maximizeButton", onSlotchange: this.updateTemplates }), h("slot", { name: "minimizeButton", onSlotchange: this.updateTemplates }), h("slot", { name: "pinButton", onSlotchange: this.updateTemplates }), h("slot", { name: "unpinButton", onSlotchange: this.updateTemplates }), h("slot", { name: "moreOptionsButton", onSlotchange: this.updateTemplates }), h("slot", { name: "splitterHandle", onSlotchange: this.updateTemplates })));
  }
  renderContentPane(pane, isFloating, isFlyout) {
    const parentPane = this.service.getParent(pane);
    let forceDrag = false;
    let isSingleFloatingContentPane = false;
    let floatingPane;
    if (isFloating) {
      floatingPane = this.service.getRootParent(pane);
      const hasHeader = this.service.hasFloatingPaneHeader(floatingPane);
      if (!hasHeader) {
        isSingleFloatingContentPane = true;
        if (this.service.forceDragPane === floatingPane) {
          this.service.forceDragPane = null;
          forceDrag = true;
        }
      }
    }
    const maximized = this.maximizedPane === pane ||
      (parentPane.type === IgcDockManagerPaneType.tabGroupPane && this.maximizedPane === parentPane) ||
      isFloating && isSingleFloatingContentPane && floatingPane === this.maximizedPane;
    return (h("igc-content-pane-component", { key: pane.id, contentId: pane.contentId, size: pane.size, isFlyout: isFlyout, unpinnedSize: pane.unpinnedSize, disabled: pane.disabled, isSingleFloating: isSingleFloatingContentPane, onMouseMove: this.togglePaneHeaderVisibility.bind(this, pane), onMouseLeave: this.removeHoveredPane.bind(this), parentUseFixedSize: this.service.isParentUseFixedSize(pane), ref: el => {
        if (el) {
          this.panesElementMap.set(pane, el);
          this.contentPanesElementMap.set(pane, el);
        }
      }, class: {
        'maximized': this.maximizedPane === pane
      }, header: pane.header, onFocusin: this.handleContentPaneFocus.bind(this, pane) }, h("igc-pane-header-component", { slot: "header", pane: pane, isActive: pane === this.activePane, disabled: pane.disabled && !isSingleFloatingContentPane, pinned: this.service.getActualIsPinned(pane), allowMaximize: this.resolveAllowMaximize(pane), maximized: maximized, allowClose: this.service.getActualAllowClose(pane), allowPinning: pane.allowPinning !== false, isFloating: isFloating, forcedDrag: forceDrag, resourceStrings: this.resourceStrings, onPinToggle: this.handlePinToggle.bind(this, pane), onMaximize: this.handleMaximize.bind(this, pane), onDragMoved: this.handlePaneDragMove.bind(this), onDragStarted: this.handlePaneDragStart.bind(this, isSingleFloatingContentPane ? floatingPane : pane), onDragEnded: this.handlePaneDragEnd.bind(this), onClose: this.handlePaneClose.bind(this, pane), onElementConnected: this.emitPaneHeaderConnected.bind(this, pane), onElementDisconnected: this.emitPaneHeaderDisconnected.bind(this, pane), onMouseDown: this.handlePaneHeaderMouseDown.bind(this, pane), class: {
        'hidden': this.showPaneHeaders === 'onHoverOnly' && this.hoveredPane !== pane,
        'transitioned': this.showPaneHeaders === 'onHoverOnly' && this.hoveredPane === pane
      } }, (isFloating && pane.floatingHeaderId) ? h("slot", { name: pane.floatingHeaderId }) :
      pane.headerId ? h("slot", { name: pane.headerId }) :
        pane.header), h("div", { class: "content", onMouseDown: this.handlePaneContentMouseDown.bind(this, pane), onScroll: this.handlePaneContentScroll.bind(this, pane) }, h("slot", { name: pane.contentId }))));
  }
  renderSplitter(parentPane, pane) {
    return (h("igc-splitter-component", { splitPaneOrientation: parentPane.orientation, onResizeStart: this.handleSplitterResizeStart.bind(this, pane), onResizeEnd: this.handleSplitterResizeEnd.bind(this, parentPane, pane), onFocusin: this.clearActivePane.bind(this), parentUseFixedSize: parentPane.useFixedSize }));
  }
  renderDocumentHost(docHost) {
    return (h("igc-document-host-component", { key: docHost.id, size: docHost.size, parentUseFixedSize: this.service.isParentUseFixedSize(docHost), ref: el => {
        if (el) {
          this.panesElementMap.set(docHost, el);
        }
      } }, this.renderSplitPane(docHost.rootPane, false, true)));
  }
  renderTabGroup(pane, isFloating, isInDocumentHost) {
    let tabs = pane.type === IgcDockManagerPaneType.tabGroupPane ? pane.panes : [pane];
    tabs = tabs.filter(t => this.service.isContentPaneVisible(t));
    const selectedIndex = pane.type === IgcDockManagerPaneType.tabGroupPane && pane.selectedIndex ?
      pane.selectedIndex : 0;
    const position = isInDocumentHost ? IgcTabHeadersPosition.top : IgcTabHeadersPosition.bottom;
    const isSingleTab = tabs.length === 1 && position === IgcTabHeadersPosition.bottom;
    const allowEmpty = pane.type === IgcDockManagerPaneType.tabGroupPane ? pane.allowEmpty : false;
    const allowMaximize = this.resolveAllowMaximize(pane);
    let contentIds = [];
    if (pane.type === IgcDockManagerPaneType.tabGroupPane) {
      pane.panes.forEach(element => {
        contentIds.push(element.contentId);
      });
    }
    else {
      contentIds.push(pane.contentId);
    }
    return (allowEmpty || tabs.length > 0) && (h("igc-tabs-component", { key: pane.id, contentIds: contentIds, size: pane.size, selectedIndex: selectedIndex, hasHeaders: !isSingleTab, parentUseFixedSize: this.service.isParentUseFixedSize(pane), maximized: this.maximizedPane === pane, allowMaximize: position === IgcTabHeadersPosition.top && allowMaximize, onMaximizeMinimizeFocus: this.clearActivePane.bind(this), onMaximize: this.handleMaximize.bind(this, pane), onSelectedIndexChanged: this.handleTabSelectedIndexChanged.bind(this, pane), onHiddenTabSelected: this.handleHiddenTabSelected.bind(this, pane), onSelectedTabOutOfView: this.handleSelectedTabOutOfView.bind(this, pane), onRendered: this.handleTabsRendered.bind(this, pane), tabHeadersPosition: position, resourceStrings: this.resourceStrings, ref: el => {
        if (el) {
          this.panesElementMap.set(pane, el);
        }
      }, class: {
        'maximized': this.maximizedPane === pane,
      } }, isSingleTab ?
      this.renderTabPanel(tabs[0], false, isFloating) :
      tabs.map(p => {
        return [
          this.renderTabHeader(p, position, isFloating),
          this.renderTabPanel(p, isInDocumentHost, isFloating)
        ];
      })));
  }
  renderTabPanel(pane, isInDocumentHost, isFloating) {
    return (h("igc-tab-panel-component", { key: pane.id, disabled: pane.disabled, onMouseDown: isInDocumentHost ? this.handlePaneContentMouseDown.bind(this, pane) : null, onSelectedChanged: this.handleTabSelectedChanged.bind(this, pane), onScroll: isInDocumentHost ? this.handlePaneContentScroll.bind(this, pane) : null }, isInDocumentHost ?
      h("slot", { name: pane.contentId }) :
      this.renderContentPane(pane, isFloating, false)));
  }
  renderTabHeader(pane, position, isFloating) {
    let forceDrag = false;
    const allowClose = this.service.getActualAllowClose(pane);
    const allowPinning = pane.allowPinning !== false;
    const iconName = isFloating ?
      allowClose ? 'close' : null :
      position === IgcTabHeadersPosition.bottom ?
        allowClose || allowPinning ? 'arrow_drop_down' : null :
        allowClose ? 'close' : null;
    if (this.service.forceDragTabHeader === pane) {
      this.service.forceDragTabHeader = null;
      forceDrag = true;
    }
    return (h("igc-tab-header-component", { key: pane.id, slot: "tabHeader", isActive: pane === this.activePane, disabled: pane.disabled, showHeaderIconOnHover: this.showHeaderIconOnHover, header: pane.header, resourceStrings: this.resourceStrings, onDragStarted: this.handleTabHeaderDragStart.bind(this, pane), onDragMoved: this.handleTabHeaderDragMove.bind(this, pane), onDragEnded: this.handleTabHeaderDragEnd.bind(this, pane), onMouseDown: this.handleTabHeaderMouseDown.bind(this, pane), onIconKeyDown: this.handleTabIconKeyDown.bind(this, iconName, pane), position: position, onIconClicked: this.handleTabIconClick.bind(this, pane, position, isFloating), iconName: iconName, forcedDrag: forceDrag, onElementConnected: this.emitTabHeaderConnected.bind(this, pane), onElementDisconnected: this.emitTabHeaderDisconnected.bind(this, pane), ref: el => {
        if (el) {
          this.tabHeadersMap.set(pane, el);
        }
      }, onFocusin: this.handleTabHeaderFocus.bind(this, pane) }, pane.tabHeaderId ?
      h("slot", { name: pane.tabHeaderId }) :
      h("span", { part: "header-title", title: pane.header }, pane.header)));
  }
  renderSplitPane(splitPane, isFloating, isInDocumentHost) {
    const panes = this.service.getSplitPaneVisibleChildren(splitPane);
    return (splitPane.allowEmpty || panes.length > 0) && (h("igc-split-pane-component", { key: splitPane.id, orientation: splitPane.orientation, size: splitPane.size, useFixedSize: splitPane.useFixedSize, parentUseFixedSize: this.service.isParentUseFixedSize(splitPane), onRendered: this.handleSplitPaneRendered.bind(this, panes), ref: el => {
        if (el) {
          this.panesElementMap.set(splitPane, el);
        }
      } }, panes.map((p, i) => {
      let paneComponent;
      if (p.type === IgcDockManagerPaneType.splitPane) {
        paneComponent = this.renderSplitPane(p, isFloating, isInDocumentHost);
      }
      else if (p.type === IgcDockManagerPaneType.contentPane) {
        paneComponent = p.contentId === "placeholder" ?
          this.renderPlaceholder(p) :
          isInDocumentHost ?
            this.renderTabGroup(p, isFloating, true) :
            this.renderContentPane(p, isFloating, false);
      }
      else if (p.type === IgcDockManagerPaneType.documentHost) {
        paneComponent = this.renderDocumentHost(p);
      }
      else if (p.type === IgcDockManagerPaneType.tabGroupPane) {
        paneComponent = this.renderTabGroup(p, isFloating, isInDocumentHost);
      }
      return i > 0 ?
        [this.renderSplitter(splitPane, p), paneComponent] :
        paneComponent;
    })));
  }
  renderPlaceholder(pane) {
    return (h("igc-content-pane-component", { key: pane.id, contentId: pane.contentId, size: pane.size }));
  }
  renderUnpinnedTabArea(location) {
    const panes = [];
    const isHorizontal = location === IgcUnpinnedLocation.top || location === IgcUnpinnedLocation.bottom;
    const isLeft = location === IgcUnpinnedLocation.left;
    const isRight = location === IgcUnpinnedLocation.right;
    const isBottom = location === IgcUnpinnedLocation.bottom;
    this.service.unpinnedLocationMap.forEach((l, p) => {
      if (l === location && p.hidden !== true) {
        panes.push(p);
      }
    });
    const tabAreaClasses = {
      'unpinned-tab-area': true,
      'unpinned-tab-area--left': isLeft,
      'unpinned-tab-area--right': isRight,
      'unpinned-tab-area--bottom': isBottom,
      'unpinned-tab-area--horizontal': isHorizontal,
      'unpinned-tab-area--hidden': panes.length === 0
    };
    return (h("div", { class: tabAreaClasses, part: Utils.partNameMap(tabAreaClasses), role: "tablist", "aria-orientation": isHorizontal ? 'horizontal' : 'vertical' }, panes.map(p => (h("igc-unpinned-pane-header-component", { location: location, isActive: p === this.activePane, disabled: p.disabled, onMouseDown: this.handleUnpinnedTabMouseDown.bind(this, p), onKeyDown: this.handleUnpinnedTabKeyDown.bind(this, p), onFocus: this.handleUnpinnedHeaderFocus.bind(this, p), ref: el => {
        if (el) {
          const slot = el.querySelector('slot');
          if (slot && p.unpinnedHeaderId) {
            slot.name = p.unpinnedHeaderId;
          }
          this.unpinnedHeadersMap.set(p, el);
        }
      } }, p.unpinnedHeaderId ? h("slot", { name: p.unpinnedHeaderId }) : p.header)))));
  }
  renderFlyoutPane() {
    const location = this.service.unpinnedLocationMap.get(this.flyoutPane);
    const flyoutClasses = {
      'flyout-pane': true,
      'flyout-pane--right': location === IgcUnpinnedLocation.right,
      'flyout-pane--bottom': location === IgcUnpinnedLocation.bottom,
      'flyout-pane--left': location === IgcUnpinnedLocation.left,
    };
    let splitPaneOrientation = IgcSplitPaneOrientation.vertical;
    if (location === IgcUnpinnedLocation.left || location === IgcUnpinnedLocation.right) {
      splitPaneOrientation = IgcSplitPaneOrientation.horizontal;
    }
    const maximized = this.maximizedPane === this.flyoutPane;
    return this.flyoutPane && (h("div", { class: flyoutClasses, style: { 'z-index': maximized ? '10002' : '2' } }, this.renderContentPane(this.flyoutPane, false, true), h("igc-splitter-component", { flyoutLocation: location, splitPaneOrientation: splitPaneOrientation, onResizeStart: this.handleSplitterResizeStart.bind(this, this.flyoutPane), onResizeEnd: this.handleFlyoutSplitterResizeEnd.bind(this, this.flyoutPane, splitPaneOrientation) })));
  }
  renderFloatingPanes() {
    var _a, _b;
    return (h("div", { class: "floating-panes", style: { position: this.maximizedPane ? 'absolute' : 'fixed' } }, (_b = (_a = this.layout) === null || _a === void 0 ? void 0 : _a.floatingPanes) === null || _b === void 0 ? void 0 : _b.map(p => {
      var _a;
      const panes = this.service.getSplitPaneVisibleChildren(p);
      if (panes.length === 0) {
        return null;
      }
      const hasHeader = this.service.hasFloatingPaneHeader(p);
      let forceDrag = false;
      if (p === this.service.forceDragPane && hasHeader) {
        this.service.forceDragPane = null;
        forceDrag = true;
      }
      const containsMaximizedPane = this.maximizedPane && this.service.getRootParent(this.maximizedPane) === p;
      return (h("igc-floating-pane-component", { key: p.id, floatingId: p.id, style: { zIndex: this.floatingPaneZIndicesMap.get(p).toString() }, floatingLocation: p.floatingLocation ? p.floatingLocation : { x: 0, y: 0 }, floatingWidth: p.floatingWidth ? p.floatingWidth : IGC_DEFAULT_PANE_SIZE, floatingHeight: p.floatingHeight ? p.floatingHeight : IGC_DEFAULT_PANE_SIZE, hasHeader: hasHeader, onWndResizeStart: this.handleFloatingPaneResizeStart.bind(this, p), onWndResizeMove: this.handleFloatingPaneResizeMove.bind(this, p), onWndResizeEnd: this.handleFloatingPaneResizeEnd.bind(this, p), onMouseDown: this.handleFloatingPaneMouseDown.bind(this, p), class: {
          'maximized': this.maximizedPane === p
        }, maximized: this.maximizedPane === p || containsMaximizedPane, allowResize: (_a = p.floatingResizable) !== null && _a !== void 0 ? _a : this.allowFloatingPanesResize }, hasHeader ?
        h("igc-pane-header-component", { slot: "header", isFloating: true, isFloatingPaneHeader: true, forcedDrag: forceDrag, allowMaximize: this.resolveAllowMaximize(p), maximized: this.maximizedPane === p, resourceStrings: this.resourceStrings, onDragStarted: this.handlePaneDragStart.bind(this, p), onDragMoved: this.handlePaneDragMove.bind(this), onDragEnded: this.handlePaneDragEnd.bind(this), onClose: this.handleFloatingPaneClose.bind(this, p), onMaximize: this.handleMaximize.bind(this, p), onFocusin: (this.clearActivePane.bind(this)) }) : null, this.renderSplitPane(p, true, false)));
    })));
  }
  renderRootDockingIndicator(position) {
    return (h("igc-root-docking-indicator-component", { position: position }));
  }
  renderDockingIndicators() {
    var _a, _b, _c;
    const startPosition = this.direction !== 'rtl' ? IgcDockingIndicatorPosition.left : IgcDockingIndicatorPosition.right;
    const endPosition = this.direction !== 'rtl' ? IgcDockingIndicatorPosition.right : IgcDockingIndicatorPosition.left;
    // Only show the joystick indicator if the target drop location is an empty split pane 
    const hideJoystickIndicator = this.dropTargetPaneInfo &&
      (((_a = this.dropTargetPaneInfo.pane) === null || _a === void 0 ? void 0 : _a.type) == "documentHost" ||
        (((_b = this.dropTargetPaneInfo.pane) === null || _b === void 0 ? void 0 : _b.type) == "splitPane" && this.service.getSplitPaneVisibleChildren(this.dropTargetPaneInfo.pane).length));
    return (h("div", { class: "docking-indicators-container", style: {
        display: this.showDockingIndicators ? 'flex' : 'none'
      } }, !this.documentOnlyDrag && !this.proximityDock && this.renderRootDockingIndicator(IgcDockingIndicatorPosition.top), h("div", { style: {
        flexGrow: '1',
        display: 'flex',
        flexDirection: 'row',
        justifyContent: 'space-between'
      } }, !this.documentOnlyDrag && !this.proximityDock && this.renderRootDockingIndicator(startPosition), !this.documentOnlyDrag && !this.proximityDock && this.renderRootDockingIndicator(endPosition)), !this.documentOnlyDrag && !this.proximityDock && this.renderRootDockingIndicator(IgcDockingIndicatorPosition.bottom), this.dropTargetPaneInfo && !this.proximityDock && !hideJoystickIndicator && ((_c = this.splitterDockingIndicator) === null || _c === void 0 ? void 0 : _c.style.visibility) !== 'visible' &&
      h("igc-joystick-indicator-component", { dropTargetPaneInfo: this.dropTargetPaneInfo, documentOnlyDrag: this.documentOnlyDrag, allowCenterDock: this.allowInnerDock }), this.allowSplitterDock && !this.proximityDock &&
      h("igc-splitter-docking-indicator-component", { style: {
          position: 'fixed',
          zIndex: '10010',
          visibility: 'hidden'
        } })));
  }
  renderContextMenu() {
    return this.contextMenuMeta && (h("igc-context-menu-component", { onMenuClosed: this.handleContextMenuClosed.bind(this), orientation: IgcContextMenuOrientation.bottom, items: this.contextMenuMeta.menuItems, target: this.contextMenuMeta.target, position: this.contextMenuMeta.position }));
  }
  renderDropShadow() {
    var _a, _b, _c, _d;
    return this.dropShadowRect && (h("div", { part: "docking-preview", class: "drop-shadow", style: {
        top: `${(_a = this.dropShadowRect) === null || _a === void 0 ? void 0 : _a.y}px`,
        left: `${(_b = this.dropShadowRect) === null || _b === void 0 ? void 0 : _b.x}px`,
        width: `${(_c = this.dropShadowRect) === null || _c === void 0 ? void 0 : _c.width}px`,
        height: `${(_d = this.dropShadowRect) === null || _d === void 0 ? void 0 : _d.height}px`,
      } }));
  }
  handlePaneNavigatorClosed(ev) {
    this.service.normalizeMaximizedPane(ev.detail);
    const parent = this.service.getRootParent(ev.detail);
    const isParentFloating = this.service.isFloatingPane(parent);
    if (isParentFloating) {
      this.service.bringFloatingPaneOnTop(parent);
    }
    if (ev.detail && this.activePane !== ev.detail) {
      this.activePane = ev.detail;
    }
    this.navigationPaneMeta = null;
  }
  renderPaneNavigator() {
    return this.navigationPaneMeta &&
      h("igc-pane-navigator-component", { onClosed: this.handlePaneNavigatorClosed.bind(this), activePanes: this.navigationPaneMeta.activePanes, activeDocuments: this.navigationPaneMeta.activeDocuments, selectedIndex: this.navigationPaneMeta.initialIndex, previousActivePaneIndex: this.navigationPaneMeta.previousActivePaneIndex, resourceStrings: this.resourceStrings });
  }
  render() {
    var _a;
    this.panesElementMap = new TwoWayMap();
    this.tabHeadersMap = new Map();
    this.unpinnedHeadersMap = new Map();
    this.contentPanesElementMap = new Map();
    return (h(Host, { tabindex: "0", role: "group" }, this.renderButtonsTemplates(), this.renderUnpinnedTabArea(IgcUnpinnedLocation.left), h("div", { class: "pane-container--vertical", style: { position: this.maximizedPane ? 'absolute' : 'relative' } }, this.renderUnpinnedTabArea(IgcUnpinnedLocation.top), h("div", { ref: el => this.dockedPanesContainer = el, class: "pane-container--horizontal", style: { position: this.maximizedPane ? 'absolute' : 'relative' } }, ((_a = this.layout) === null || _a === void 0 ? void 0 : _a.rootPane) && this.renderSplitPane(this.layout.rootPane, false, false), this.renderFlyoutPane(), this.renderDockingIndicators()), this.renderUnpinnedTabArea(IgcUnpinnedLocation.bottom)), this.renderUnpinnedTabArea(IgcUnpinnedLocation.right), this.renderFloatingPanes(), this.renderContextMenu(), this.renderPaneNavigator(), h("igc-trial-watermark", { text: "Trial version of Ignite UI", style: { 'z-index': '10005' } }), this.renderDropShadow()));
  }
  static get is() { return "igc-dockmanager"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() {
    return {
      "$": ["dockmanager-component.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["dockmanager-component.css"]
    };
  }
  static get properties() {
    return {
      "navigationPaneMeta": {
        "type": "unknown",
        "mutable": true,
        "complexType": {
          "original": "IgcPaneNavigatorMetadata",
          "resolved": "IgcPaneNavigatorMetadata",
          "references": {
            "IgcPaneNavigatorMetadata": {
              "location": "import",
              "path": "./dockmanager.interfaces",
              "id": "src/components/dockmanager/dockmanager.interfaces.ts::IgcPaneNavigatorMetadata"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        }
      },
      "allowMaximize": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "allow-maximize",
        "reflect": false,
        "defaultValue": "true"
      },
      "showPaneHeaders": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "'always' | 'onHoverOnly'",
          "resolved": "\"always\" | \"onHoverOnly\"",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "show-pane-headers",
        "reflect": false,
        "defaultValue": "'always'"
      },
      "showHeaderIconOnHover": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "'none' | 'closeOnly' | 'moreOptionsOnly' | 'all'",
          "resolved": "\"all\" | \"closeOnly\" | \"moreOptionsOnly\" | \"none\"",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "show-header-icon-on-hover",
        "reflect": false,
        "defaultValue": "'none'"
      },
      "maximizedPane": {
        "type": "unknown",
        "mutable": true,
        "complexType": {
          "original": "IgcContentPane | IgcSplitPane | IgcTabGroupPane",
          "resolved": "IgcContentPane | IgcSplitPane | IgcTabGroupPane",
          "references": {
            "IgcContentPane": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcContentPane"
            },
            "IgcSplitPane": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcSplitPane"
            },
            "IgcTabGroupPane": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcTabGroupPane"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        }
      },
      "resourceStrings": {
        "type": "unknown",
        "mutable": true,
        "complexType": {
          "original": "IgcDockManagerResourceStrings",
          "resolved": "IgcDockManagerResourceStrings",
          "references": {
            "IgcDockManagerResourceStrings": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcDockManagerResourceStrings"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        }
      },
      "allowFloatingPanesResize": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "allow-floating-panes-resize",
        "reflect": false,
        "defaultValue": "true"
      },
      "disableKeyboardNavigation": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "disable-keyboard-navigation",
        "reflect": false,
        "defaultValue": "false"
      },
      "contextMenuPosition": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "IgcContextMenuPosition",
          "resolved": "\"center\" | \"end\" | \"start\" | \"stretch\"",
          "references": {
            "IgcContextMenuPosition": {
              "location": "import",
              "path": "./dockmanager.interfaces",
              "id": "src/components/dockmanager/dockmanager.interfaces.ts::IgcContextMenuPosition"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "context-menu-position",
        "reflect": false
      },
      "containedInBoundaries": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "contained-in-boundaries",
        "reflect": false,
        "defaultValue": "false"
      },
      "allowInnerDock": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "allow-inner-dock",
        "reflect": false,
        "defaultValue": "true"
      },
      "proximityDock": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "proximity-dock",
        "reflect": false,
        "defaultValue": "false"
      },
      "closeBehavior": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "IgcPaneActionBehavior",
          "resolved": "IgcPaneActionBehavior.allPanes | IgcPaneActionBehavior.selectedPane",
          "references": {
            "IgcPaneActionBehavior": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcPaneActionBehavior"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "close-behavior",
        "reflect": false,
        "defaultValue": "IgcPaneActionBehavior.allPanes"
      },
      "unpinBehavior": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "IgcPaneActionBehavior",
          "resolved": "IgcPaneActionBehavior.allPanes | IgcPaneActionBehavior.selectedPane",
          "references": {
            "IgcPaneActionBehavior": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcPaneActionBehavior"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "unpin-behavior",
        "reflect": false,
        "defaultValue": "IgcPaneActionBehavior.allPanes"
      },
      "allowSplitterDock": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "allow-splitter-dock",
        "reflect": false,
        "defaultValue": "false"
      },
      "layout": {
        "type": "unknown",
        "mutable": true,
        "complexType": {
          "original": "IgcDockManagerLayout",
          "resolved": "IgcDockManagerLayout",
          "references": {
            "IgcDockManagerLayout": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcDockManagerLayout"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "The layout configuration of the Dock Manager."
        }
      },
      "draggedPane": {
        "type": "unknown",
        "mutable": true,
        "complexType": {
          "original": "IgcContentPane | IgcSplitPane | IgcTabGroupPane",
          "resolved": "IgcContentPane | IgcSplitPane | IgcTabGroupPane",
          "references": {
            "IgcContentPane": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcContentPane"
            },
            "IgcSplitPane": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcSplitPane"
            },
            "IgcTabGroupPane": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcTabGroupPane"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        }
      },
      "dropPosition": {
        "type": "unknown",
        "mutable": false,
        "complexType": {
          "original": "IgcDockManagerPoint",
          "resolved": "IgcDockManagerPoint",
          "references": {
            "IgcDockManagerPoint": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcDockManagerPoint"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        }
      },
      "activePane": {
        "type": "unknown",
        "mutable": true,
        "complexType": {
          "original": "IgcContentPane",
          "resolved": "IgcContentPane",
          "references": {
            "IgcContentPane": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcContentPane"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "defaultValue": "null"
      }
    };
  }
  static get states() {
    return {
      "dropTargetPaneInfo": {},
      "flyoutPane": {},
      "floatingPaneZIndicesMap": {},
      "contextMenuMeta": {},
      "dropShadowRect": {},
      "templates": {},
      "hasCustomMaximizeButton": {},
      "hasCustomMinimizeButton": {},
      "hoveredPane": {}
    };
  }
  static get events() {
    return [{
        "method": "paneHeaderConnected",
        "name": "paneHeaderConnected",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "IgcPaneHeaderConnectionEventArgs",
          "resolved": "IgcPaneHeaderConnectionEventArgs",
          "references": {
            "IgcPaneHeaderConnectionEventArgs": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcPaneHeaderConnectionEventArgs"
            }
          }
        }
      }, {
        "method": "paneHeaderDisconnected",
        "name": "paneHeaderDisconnected",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "IgcPaneHeaderConnectionEventArgs",
          "resolved": "IgcPaneHeaderConnectionEventArgs",
          "references": {
            "IgcPaneHeaderConnectionEventArgs": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcPaneHeaderConnectionEventArgs"
            }
          }
        }
      }, {
        "method": "tabHeaderConnected",
        "name": "tabHeaderConnected",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "IgcTabHeaderConnectionEventArgs",
          "resolved": "IgcTabHeaderConnectionEventArgs",
          "references": {
            "IgcTabHeaderConnectionEventArgs": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcTabHeaderConnectionEventArgs"
            }
          }
        }
      }, {
        "method": "tabHeaderDisconnected",
        "name": "tabHeaderDisconnected",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "IgcTabHeaderConnectionEventArgs",
          "resolved": "IgcTabHeaderConnectionEventArgs",
          "references": {
            "IgcTabHeaderConnectionEventArgs": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcTabHeaderConnectionEventArgs"
            }
          }
        }
      }, {
        "method": "splitterResizeStart",
        "name": "splitterResizeStart",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "IgcSplitterResizeEventArgs",
          "resolved": "IgcSplitterResizeEventArgs",
          "references": {
            "IgcSplitterResizeEventArgs": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcSplitterResizeEventArgs"
            }
          }
        }
      }, {
        "method": "splitterResizeEnd",
        "name": "splitterResizeEnd",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "IgcSplitterResizeEventArgs",
          "resolved": "IgcSplitterResizeEventArgs",
          "references": {
            "IgcSplitterResizeEventArgs": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcSplitterResizeEventArgs"
            }
          }
        }
      }, {
        "method": "paneClose",
        "name": "paneClose",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "IgcPaneCloseEventArgs",
          "resolved": "IgcPaneCloseEventArgs",
          "references": {
            "IgcPaneCloseEventArgs": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcPaneCloseEventArgs"
            }
          }
        }
      }, {
        "method": "paneScroll",
        "name": "paneScroll",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "IgcPaneScrollEventArgs",
          "resolved": "IgcPaneScrollEventArgs",
          "references": {
            "IgcPaneScrollEventArgs": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcPaneScrollEventArgs"
            }
          }
        }
      }, {
        "method": "panePinnedToggle",
        "name": "panePinnedToggle",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "IgcPanePinnedEventArgs",
          "resolved": "IgcPanePinnedEventArgs",
          "references": {
            "IgcPanePinnedEventArgs": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcPanePinnedEventArgs"
            }
          }
        }
      }, {
        "method": "activePaneChanged",
        "name": "activePaneChanged",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "IgcActivePaneEventArgs",
          "resolved": "IgcActivePaneEventArgs",
          "references": {
            "IgcActivePaneEventArgs": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcActivePaneEventArgs"
            }
          }
        }
      }, {
        "method": "paneDragStart",
        "name": "paneDragStart",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "IgcPaneDragStartEventArgs",
          "resolved": "IgcPaneDragStartEventArgs",
          "references": {
            "IgcPaneDragStartEventArgs": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcPaneDragStartEventArgs"
            }
          }
        }
      }, {
        "method": "paneDragOver",
        "name": "paneDragOver",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "IgcPaneDragOverEventArgs",
          "resolved": "IgcPaneDragOverEventArgs",
          "references": {
            "IgcPaneDragOverEventArgs": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcPaneDragOverEventArgs"
            }
          }
        }
      }, {
        "method": "paneDragEnd",
        "name": "paneDragEnd",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "IgcPaneDragEndEventArgs",
          "resolved": "IgcPaneDragEndEventArgs",
          "references": {
            "IgcPaneDragEndEventArgs": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcPaneDragEndEventArgs"
            }
          }
        }
      }, {
        "method": "floatingPaneResizeMove",
        "name": "floatingPaneResizeMove",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "IgcFloatingPaneResizeMoveEventArgs",
          "resolved": "IgcFloatingPaneResizeMoveEventArgs",
          "references": {
            "IgcFloatingPaneResizeMoveEventArgs": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcFloatingPaneResizeMoveEventArgs"
            }
          }
        }
      }, {
        "method": "floatingPaneResizeEnd",
        "name": "floatingPaneResizeEnd",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "IgcFloatingPaneResizeEventArgs",
          "resolved": "IgcFloatingPaneResizeEventArgs",
          "references": {
            "IgcFloatingPaneResizeEventArgs": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcFloatingPaneResizeEventArgs"
            }
          }
        }
      }, {
        "method": "floatingPaneResizeStart",
        "name": "floatingPaneResizeStart",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "IgcFloatingPaneResizeEventArgs",
          "resolved": "IgcFloatingPaneResizeEventArgs",
          "references": {
            "IgcFloatingPaneResizeEventArgs": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcFloatingPaneResizeEventArgs"
            }
          }
        }
      }, {
        "method": "layoutChange",
        "name": "layoutChange",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get methods() {
    return {
      "dropPane": {
        "complexType": {
          "signature": "() => Promise<boolean>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global",
              "id": "global::Promise"
            }
          },
          "return": "Promise<boolean>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "removePane": {
        "complexType": {
          "signature": "(pane: IgcDockManagerPane) => Promise<void>",
          "parameters": [{
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global",
              "id": "global::Promise"
            },
            "IgcDockManagerPane": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcDockManagerPane"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "focusPane": {
        "complexType": {
          "signature": "(contentId: string) => Promise<void>",
          "parameters": [{
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global",
              "id": "global::Promise"
            },
            "IgcContentPane": {
              "location": "import",
              "path": "./dockmanager.public-interfaces",
              "id": "src/components/dockmanager/dockmanager.public-interfaces.ts::IgcContentPane"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      }
    };
  }
  static get elementRef() { return "element"; }
  static get watchers() {
    return [{
        "propName": "flyoutPane",
        "methodName": "flyoutPaneChanged"
      }, {
        "propName": "containedInBoundaries",
        "methodName": "containedInBoundariesChanged"
      }, {
        "propName": "layout",
        "methodName": "layoutChanged"
      }, {
        "propName": "draggedPane",
        "methodName": "draggedPaneChanged"
      }, {
        "propName": "dropPosition",
        "methodName": "dropPositionChanged"
      }, {
        "propName": "activePane",
        "methodName": "activePaneChange"
      }];
  }
  static get listeners() {
    return [{
        "name": "focusout",
        "method": "handleFocusOut",
        "target": undefined,
        "capture": false,
        "passive": false
      }, {
        "name": "keydown",
        "method": "handleKeyDown",
        "target": undefined,
        "capture": false,
        "passive": false
      }, {
        "name": "pointerdown",
        "method": "handlePointerDown",
        "target": undefined,
        "capture": false,
        "passive": true
      }, {
        "name": "pointerup",
        "method": "handlePointerUp",
        "target": undefined,
        "capture": false,
        "passive": true
      }];
  }
}
//# sourceMappingURL=dockmanager-component.js.map
